#!/usr/bin/env python

# Copyright 2018 Earth Sciences Department, BSC-CNS
#
# This file is part of HERMESv3_GR.
#
# HERMESv3_GR is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HERMESv3_GR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HERMESv3_GR. If not, see <http://www.gnu.org/licenses/>.

import os
import sys
from timeit import default_timer as gettime

import hermesv3_gr.config.settings as settings
from emission_inventory import EmissionInventory


class GfasEmissionInventory(EmissionInventory):
    """
    Class that defines the content and the methodology for the GFAS emission inventories

    :param current_date: Date of the day to simulate.
    :type current_date: datetime.datetime

    :param inventory_name: Name of the inventory to use.
    :type inventory_name: str

    :param sector: Name of the sector of the inventory to use.
    :type sector: str

    :param pollutants: List of the pollutant name to take into account.
    :type pollutants: list of str

    :param frequency: Frequency of the inputs. [yearly, monthly, daily]
    :type frequency: str

    :param reference_year: year of reference of the information of the dataset.
    :type reference_year: int

    :param factors: NOT IMPLEMENTED YET
    :type factors: NOT IMPLEMENTED YET

    :param regrid_mask: NOT IMPLEMENTED YET
    :type regrid_mask: NOT IMPLEMENTED YET

    :param p_vertical: ID of the vertical profile to use.
    :type p_vertical: str

    :param p_month: ID of the temporal monthly profile to use.
    :type p_month: str

    :param p_day: ID of the temporal daily profile to use.
    :type p_day: str

    :param p_hour: ID of the temporal hourly profile to use.
    :type p_hour: str

    :param p_speciation: ID of the speciation profile to use.
    :type p_speciation: str
    """

    def __init__(self, options, grid, current_date, inventory_name, source_type, sector, pollutants, inputs_path,
                 frequency, vertical_output_profile,
                 reference_year=2010, factors=None, regrid_mask=None, p_vertical=None, p_month=None, p_day=None,
                 p_hour=None, p_speciation=None):
        from hermesv3_gr.modules.vertical.vertical_gfas import GfasVerticalDistribution

        st_time = gettime()
        settings.write_log('\t\tCreating GFAS emission inventory.', level=3)

        super(GfasEmissionInventory, self).__init__(
            options, grid, current_date, inventory_name, source_type, sector, pollutants, inputs_path, frequency,
            vertical_output_profile,
            reference_year=reference_year, factors=factors, regrid_mask=regrid_mask, p_vertical=None,
            p_month=p_month, p_day=p_day, p_hour=p_hour, p_speciation=p_speciation)

        self.approach = self.get_approach(p_vertical)
        self.method = self.get_method(p_vertical)

        # self.altitude = self.get_altitude()

        self.vertical = GfasVerticalDistribution(vertical_output_profile, self.approach, self.get_altitude())

        settings.write_time('GFAS_EmissionInventory', 'Init', gettime() - st_time, level=3)

        return None

    def get_input_path(self, pollutant=None, extension='nc'):
        """
        Completes the path of the NetCDF that contains the needed information of the given pollutant.

        :param pollutant: Name of the pollutant of the NetCDF.
        :type pollutant: str

        :return: Full path of the needed NetCDF.
        :rtype: str
        """
        st_time = gettime()

        netcdf_path = os.path.join(self.inputs_path, 'multivar', 'ga_{0}.{1}'.format(
            self.date.strftime('%Y%m%d'), extension))

        settings.write_time('GfasEmissionInventory', 'get_input_path', gettime() - st_time, level=3)

        return netcdf_path

    def get_altitude(self):
        """
        Extracts the altitude values depending on the choosen method.

        :return: Array with the alittude of each fire.
        :rtype: numpy.ndarray
        """
        from hermesv3_gr.tools.netcdf_tools import extract_vars

        st_time = gettime()

        if self.method == 'sovief':
            alt_var = 'apt'
        elif self.method == 'prm':
            alt_var = 'mami'
        else:
            alt_var = None

            print "ERROR: Only 'sovief' and 'prm' methods are accepted."

        [alt] = extract_vars(self.get_input_path(), [alt_var])

        alt = alt['data']

        settings.write_time('GfasEmissionInventory', 'get_altitude', gettime() - st_time, level=3)
        return alt

    def get_approach(self, p_vertical):
        """
        Extracts the given approach value.

        :return: Approach value
        :rtype: str
        """
        import re

        st_time = gettime()

        return_value = None
        aux_list = re.split(', |,| , | ,', p_vertical)
        for element in aux_list:
            aux_value = re.split('=| =|= | = ', element)
            if aux_value[0] == 'approach':
                return_value = aux_value[1]

        settings.write_time('GfasEmissionInventory', 'get_approach', gettime() - st_time, level=3)

        return return_value

    def get_method(self, p_vertical):
        """
        Extracts the given method value.

        :return: Method value
        :rtype: str
        """
        import re

        st_time = gettime()

        return_value = None
        aux_list = re.split(', |,| , | ,', p_vertical)
        for element in aux_list:
            aux_value = re.split('=| =|= | = ', element)
            if aux_value[0] == 'method':
                return_value = aux_value[1]

        settings.write_time('GfasEmissionInventory', 'get_method', gettime() - st_time, level=3)

        return return_value

    def do_vertical_allocation(self, values):
        """
        Allocates the fire emissions on their top level.

        :param values: 2D array with the fire emissions
        :type values: numpy.array

        :return: Emissions already allocated on the top altitude of each fire.
        :rtype: numpy.array
        """
        st_time = gettime()

        return_value = self.vertical.do_vertical_interpolation_allocation(values, self.altitude)

        settings.write_time('GfasEmissionInventory', 'do_vertical_allocation', gettime() - st_time, level=3)

        return return_value

    def do_regrid(self):

        st_time = gettime()
        settings.write_log("\tRegridding", level=2)

        for i in xrange(len(self.emissions)):
            self.emissions[i]["data"] = self.do_vertical_allocation(self.emissions[i]["data"])

        regridded_emissions = self.regrid.start_regridding(gfas=True, vertical=self.vertical)

        for emission in regridded_emissions:
            dict_aux = {'name': emission['name'], 'data': emission['data'], 'units': 'm'}
            # dict_aux['data'] = dict_aux['data'].reshape((1,) + dict_aux['data'].shape)
            self.emissions.append(dict_aux)
        self.vertical = None

        settings.write_time('GfasEmissionInventory', 'do_regrid', gettime() - st_time, level=2)

    # def do_vertical(self, values):
    #     """
    #     Distribute the emissions on the top layer into their low layers depending on the method
    #
    #     :param values: Emissions to distribute.
    #     :type values: numpy.ndarray
    #
    #     :return: Emission already distributed.
    #     :rtype: numpy.ndarray
    #     """
    #     if settings.log_level_3:
    #         st_time = gettime()
    #     else:
    #         st_time = None
    #     if settings.log_level_1:
    #         print "\tVertical interpolation (Inventory: {0}; Sector: {1})".format(self.inventory_name, self.sector)
    #
    #     return_value = self.vertical.do_vertical_interpolation(values)
    #
    #     if settings.log_level_3:
    #         print "TIME -> EmissionInventory.do_vertical: {0} s".format(round(gettime() - st_time, 2))
    #
    #     return return_value
    #
    # def process_emissions(self, auxiliary_files_path, grid_type, vertical_description_path, vertical_profile_path,
    #                       p_month, p_day, p_hour, date, timestep_type, timestep_num, timestep_freq, world_info,
    #                       speciation_profile_path, molecular_weights_path, fluxes=False,
    #                       inc_lat=None, inc_lon=None,
    #                       new_pole_longitude_degrees=None, new_pole_latitude_degrees=None, centre_lat=None, centre_lon=None,
    #                       west_boundary=None, south_boundary=None, inc_rlat=None, inc_rlon=None,
    #                       lat_1=None, lat_2=None, lon_0=None, lat_0=None, nx=None, ny=None, inc_x=None, inc_y=None,
    #                       x_0=None, y_0=None,
    #                       is_nmmb=False):
    #     """
    #     Manages all the process to get the desired output.
    #
    #     :param auxiliary_files_path: Path to the folder where will be stored the weight matrix files if them are not created yet.
    #     :type auxiliary_files_path: str
    #
    #     :param grid_type: Type of the destination grid
    #     :type grid_type: str
    #
    #     :param inc_lat: Resolution of the latitude coordinates.
    #     :type inc_lat: float
    #
    #     :param inc_lon: Resolution of the longitude coordinates.
    #     :type inc_lon: float
    #
    #     :param vertical_profile_path: Path to the file that contains all the vertical profiles.
    #     :type vertical_profile_path: str
    #
    #     :param vertical_description_path: path to the file that contain the vertical description of the required output
    #     file.
    #     :type vertical_description_path: str
    #
    #     :param speciation_profile_path: Path to the file that contains all the speciation profiles.
    #     :type speciation_profile_path: str
    #
    #     :param molecular_weights_path: Path to the file that contains all the needed molecular weights.
    #     :type molecular_weights_path: str
    #
    #     :param is_nmmb: Indicates if the required ouptut is for NMMB-MONARCH
    #     :type is_nmmb: bool
    #     """
    #     from hermesv3_gr.modules.vertical.vertical_gfas import GfasVerticalDistribution
    #     from hermesv3_gr.modules.regrid.regrid_conservative import ConservativeRegrid
    #
    #     if settings.log_level_3:
    #         st_time = gettime()
    #     else:
    #         st_time = None
    #
    #     self.emissions = self.read_emissions()
    #     self.masking.check_regrid_mask(self.get_input_path(pollutant=self.pollutant_dicts[0]))
    #
    #     self.vertical = GfasVerticalDistribution(vertical_description_path, self.approach)
    #
    #     weight_matrix_file = os.path.join(auxiliary_files_path, "Weight_Matrix_{0}.nc".format(self.inventory_name))
    #     self.regrid = ConservativeRegrid(self.get_input_path(), weight_matrix_file, grid_type, auxiliary_files_path,
    #                                      masking=self.masking, inc_lat=inc_lat, inc_lon=inc_lon,
    #                                      centre_lat=centre_lat,
    #                                      centre_lon=centre_lon, west_boundary=west_boundary,
    #                                      south_boundary=south_boundary, inc_rlat=inc_rlat, inc_rlon=inc_rlon,
    #                                      lat_1=lat_1, lat_2=lat_2, lon_0=lon_0, lat_0=lat_0, nx=nx, ny=ny, inc_x=inc_x,
    #                                      inc_y=inc_y, x_0=x_0, y_0=y_0,
    #                                      is_nmmb=is_nmmb)
    #
    #     for i in xrange(len(self.emissions)):
    #         self.emissions[i]["data"] = self.do_vertical_allocation(self.emissions[i]["data"])
    #         self.emissions[i]["data"] = self.do_regrid(self.emissions[i]["data"])
    #         self.emissions[i]["data"] = self.do_vertical(self.emissions[i]["data"])
    #     self.do_temporal(date, timestep_type, timestep_num, timestep_freq, p_month, p_day, p_hour, world_info,
    #                      auxiliary_files_path, self.regrid.grid)
    #     self.do_speciation(speciation_profile_path, molecular_weights_path, self.regrid.grid.cell_area, fluxes)
    #     if is_nmmb and self.regrid.grid.grid_type == 'global':
    #         self.unify_borders()
    #         self.add_ghost_cell()
    #
    #     if settings.log_level_3:
    #         print "TIME -> EmissionInventory.process_emissions: {0} s".format(round(gettime() - st_time, 2))

if __name__ == "__main__":
    pass
