#!/usr/bin/env python

# Copyright 2018 Earth Sciences Department, BSC-CNS
#
# This file is part of HERMESv3_GR.
#
# HERMESv3_GR is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# HERMESv3_GR is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with HERMESv3_GR. If not, see <http://www.gnu.org/licenses/>.


import os
from timeit import default_timer as gettime
import sys
import numpy as np
from netCDF4 import Dataset
from mpi4py import MPI
from hermesv3_gr.config import settings


def open_netcdf(netcdf_path):
    from netCDF4 import Dataset
    nc_out = Dataset(netcdf_path, mode='a')
    return nc_out


def close_netcdf(nc):
    nc.close()


def get_grid_area(filename):
    """
    Calculates the area of each cell.

    :param filename: Full path to the NetCDF to calculate the cell areas.
    :type filename: str

    :return: Returns the area of each cell.
    :rtype: numpy.ndarray
    """
    from cdo import Cdo

    cdo = Cdo()
    s = cdo.gridarea(input=filename)
    nc_aux = Dataset(s, mode='r')
    grid_area = nc_aux.variables['cell_area'][:]
    nc_aux.close()

    return grid_area


def extract_vars(netcdf_path, variables_list, attributes_list=[]):
    data_list = []
    # print netcdf_path
    netcdf = Dataset(netcdf_path, mode='r')
    for var in variables_list:
        if var == 'emi_nox_no2':
            var1 = var
            var2 = 'emi_nox'
        else:
            var1 = var2 = var
        dict_aux = \
            {
                'name': var1,
                'data': netcdf.variables[var2][:],
            }
        for attribute in attributes_list:
            dict_aux.update({attribute: netcdf.variables[var2].getncattr(attribute)})
        data_list.append(dict_aux)
    netcdf.close()

    return data_list



def write_netcdf(netcdf_path, center_latitudes, center_longitudes, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None,
                 Mercator=False, lat_ts=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    if not (RegularLatLon or LambertConformalConic or Rotated or Mercator):
        RegularLatLon = True
    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(center_latitudes.shape) == 1:
            netcdf.createDimension('lat', center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(center_latitudes.shape) == 2:
            netcdf.createDimension('lat', center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(center_longitudes.shape) == 1:
            netcdf.createDimension('lon', center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(center_longitudes.shape) == 2:
            netcdf.createDimension('lon', center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)
    elif LambertConformalConic or Mercator:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        try:
            netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        except TypeError:
            netcdf.createDimension('nv', boundary_latitudes.shape[1])

        # sys.exit()
    
    # Time
    netcdf.createDimension('time', None)

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    lats[:] = center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=True)

    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    # print 'lons:', lons[:].shape, center_longitudes.shape
    lons[:] = center_longitudes
    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        rlon[:] = rotated_lons
    if LambertConformalConic or Mercator:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=True)
        var[:] = 0
    for variable in data_list:
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        elif Mercator:
            var.grid_mapping = 'mercator'
        try:
            var[:] = variable['data']
        except:
            print 'VAR ERROR, netcdf shape: {0}, variable shape: {1}'.format(var[:].shape, variable['data'].shape)

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0
    elif Mercator:
        #Mercator
        mapping = netcdf.createVariable('mercator', 'i')
        mapping.grid_mapping_name = "mercator"
        mapping.longitude_of_projection_origin = lon_0
        mapping.standard_parallel = lat_ts

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[:] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()

def write_netcdf_parallel(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    netcdf.createDimension('time', None)

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=True)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=True)
        var[:] = 0
    for variable in data_list:
        print "Rank {0} var: {1}".format(rank, variable['name'])
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()


def write_simple_netcdf_parallel(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    from cf_units import Unit, encode_time

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())

    # ===== Dimensions =====
    var_dim = ('lat', 'lon',)
    netcdf.createDimension('lat', grid.center_latitudes.shape[0])
    # lat_dim = ('lat',)
    netcdf.createDimension('lon', grid.center_longitudes.shape[0])
    # lon_dim = ('lon',)
    netcdf.createDimension('lev', len(levels))
    # netcdf.createDimension('time', None)
    netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    time = netcdf.createVariable('time', 'd', ('time',))#, zlib=True)
    u = Unit('hours')
    time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
    time.standard_name = "time"
    time.calendar = "gregorian"
    time.long_name = "time"
    if rank == 0:
        time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', 'lat')# , zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes


    # Longitude
    lons = netcdf.createVariable('lon', 'f', 'lon')# , zlib=True)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    cell_area_dim = var_dim
    # Levels
    var_dim = ('lev',) + var_dim
    lev = netcdf.createVariable('lev', 'f', ('lev',))# , zlib=True)
    lev.units = Unit("m").symbol
    lev.positive = 'up'
    if rank == 0:
        lev[:] = levels

    index = 0
    for variable in data_list:
        index += 1
        print "Rank {0} var: {1}; {2}/{3}".format(rank, variable['name'], index,len(data_list))

        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim)
                                    # chunksizes=(1, len(levels),
                                    #             grid.x_upper_bound - grid.x_lower_bound,
                                    #             grid.y_upper_bound - grid.y_lower_bound))# , zlib=True)
        var.units = Unit(variable['units']).symbol
        # print "Rank {0} in: {1}, out: {2}".format(rank, variable['data'].shape, var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound].shape)
    netcdf.close()
    netcdf = Dataset(netcdf_path, mode='r+', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())
    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        print "Rank {0} var: {1}; {2}/{3}".format(rank, variable['name'], index,len(data_list))

        var = netcdf.variables[variable['name']]
        if index == 8:
            pass    
            # print variable['data'].shape, var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound].shape
        else:
            var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        print "Rank {0} var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    netcdf.close()

def write_chunked_netcdf_parallel(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    # netcdf.createDimension('time', None)
    netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=True)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=True)
        var[:] = 0

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        print "Rank {0} var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim,
                                    chunksizes=(1, len(levels),
                                                grid.x_upper_bound - grid.x_lower_bound,
                                                grid.y_upper_bound - grid.y_lower_bound), zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        settings.comm.Barrier()

        print "Rank {0} var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()

def write_chunked_uncompressed_netcdf_parallel(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    # netcdf.createDimension('time', None)
    netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim)#, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',))#, zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim)#, zlib=True)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',))#, zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',))#, zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',))#, zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',))#, zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',))#, zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',))#, zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim)#, zlib=True)
        var[:] = 0

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        print "Rank {0} var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim,
                                    chunksizes=(1, len(levels),
                                                grid.x_upper_bound - grid.x_lower_bound,
                                                grid.y_upper_bound - grid.y_lower_bound))# , zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        settings.comm.Barrier()

        print "Rank {0} var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()

def only_create_chunked_uncompressed_netcdf_parallel(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    compressed = False

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    # netcdf.createDimension('time', None)
    netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=compressed)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=compressed)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=compressed)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=compressed)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=compressed)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=compressed)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=compressed)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=compressed)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=compressed)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=compressed)
        var[:] = 0

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        print "Rank {0} creating var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim,
                                    chunksizes=(1, len(levels),
                                                grid.x_upper_bound - grid.x_lower_bound,
                                                grid.y_upper_bound - grid.y_lower_bound), zlib=compressed)
        # print '1'

        var.units = Unit(variable['units']).symbol
        # print '2'
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        # print '3'
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        # print '4'
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        # print '5'
        var.coordinates = "lat lon"
        # print '6'
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # print '8'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        # var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        print "Rank {0} created var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        # c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()

def only_write_chunked_uncompressed_netcdf_parallel(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='r+', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        # print "Rank {0} writing var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))

        var = netcdf.variables[variable['name']]
        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        # print "Rank {0} ---> 2".format(rank)
        # settings.comm.Barrier()
        print "TIME -> VarWritten Rank {0} {1} {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    c_area = netcdf.variables['cell_area']
    c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    netcdf.close()

def write_netcdf_serie(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    netcdf.createDimension('time', None)
    # netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim)#, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',))#, zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim)#, zlib=True)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',))#, zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',))#, zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',))#, zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',))#, zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',))#, zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',))#, zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim)#, zlib=True)
        var[:] = 0

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        print "Rank {0} var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim,
                                    chunksizes=(1, len(levels),
                                                grid.x_upper_bound - grid.x_lower_bound,
                                                grid.y_upper_bound - grid.y_lower_bound))# , zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        print "Rank {0} var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()


def gather_netcdf(file_list, output_path, common_vars=[]):
    c_lats, c_lons, levs = extract_vars(file_list[0], ['lat', 'lon', 'lev'])

    from timeit import default_timer as get_time

    st_time = get_time()
    data_list = []

    for file in file_list:
        nc_in = Dataset(file, mode='r')
        # variables = nc_in.variables
        for i, var in nc_in.variables.iteritems():
            if var.name not in common_vars:
                dict_aux = \
                    {
                        'name': var.name,
                        'data': var[:],
                        'units': var.units,
                    }
                data_list.append(dict_aux)

        # print type(variables)
        # print variables.popitem()
        # OrderedDict.
        nc_in.close()
    write_netcdf(output_path, c_lats['data'], c_lons['data'], data_list, levels=levs['data'])

    print 'Time of gattering netcdf: {0}'.format(round(get_time() - st_time, 2))

# @profile
def create_netcdf(netcdf_path, center_latitudes, center_longitudes, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    from cf_units import Unit, encode_time
    import sys
    from netCDF4 import Dataset
    import numpy as np

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(center_latitudes.shape) == 1:
            netcdf.createDimension('lat', center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(center_latitudes.shape) == 2:
            netcdf.createDimension('lat', center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(center_longitudes.shape) == 1:
            netcdf.createDimension('lon', center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(center_longitudes.shape) == 2:
            netcdf.createDimension('lon', center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    netcdf.createDimension('time', None)

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    lats[:] = center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=True)

    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    lons[:] = center_longitudes
    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=True)
        var[:] = 0
    for variable in data_list:
        # print ('time',) + var_dim
        # print variable
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # print 'HOURSSSSSSSSSSSSSSSSSSSSS:', hours
        # if variable['data'] is not 0:
        #     print var[:].shape, variable['data'].shape, variable['data'].max()
        shape = None
        exec ("shape = (len(hours), {0}.size, {1}.size, {2}.size)".format(var_dim[0], var_dim[1], var_dim[2]))
        # exit()
        print shape
        var[:] = np.zeros(shape)

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[:] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)
    return netcdf


def create_netcdf_close(netcdf_path, center_latitudes, center_longitudes, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    from cf_units import Unit, encode_time
    import sys
    from netCDF4 import Dataset
    import numpy as np

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(center_latitudes.shape) == 1:
            netcdf.createDimension('lat', center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(center_latitudes.shape) == 2:
            netcdf.createDimension('lat', center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(center_longitudes.shape) == 1:
            netcdf.createDimension('lon', center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(center_longitudes.shape) == 2:
            netcdf.createDimension('lon', center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    netcdf.createDimension('time', None)

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',), zlib=True)
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=True)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    lats[:] = center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=True)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=True)

    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    lons[:] = center_longitudes
    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=True)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=True)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=True)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=True)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=True)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=True)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        lev[:] = levels

    # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=True)
        var[:] = 0
    for variable in data_list:
        # print ('time',) + var_dim
        # print variable
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=True)
        var.units = Unit(variable['units']).symbol
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        var.coordinates = "lat lon"
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'
        # print 'HOURSSSSSSSSSSSSSSSSSSSSS:', hours
        # if variable['data'] is not 0:
        #     print var[:].shape, variable['data'].shape, variable['data'].max()
        shape = None
        exec ("shape = (len(hours), {0}.size, {1}.size, {2}.size)".format(var_dim[0], var_dim[1], var_dim[2]))
        # exit()
        var[:] = np.zeros(shape)

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        c_area[:] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)
    netcdf.close
    return True





def fill_netcdf(tstep, nc, data):
    import numpy as np
    print tstep

    print 'Filling time step number {0}'.format(tstep)

    for pollutant in data:
        # print pollutant
        var = nc.variables[pollutant['name']]
        # print pollutant['data'].shape
        # print var.shape
        # print 'isnan:', np.isnan(var[:].sum())
        # print 'var:', var[:].sum(), type(var[:].sum())
        # exit()
        if pollutant['data'] is not 0:
            if pollutant['data'].shape[1] is 1:
                var[tstep, 0, :, :] += pollutant['data'][0, :, :]
            else:
                print var[tstep, :, :, :].shape, pollutant['data'].shape
                var[tstep, :, :, :] += pollutant['data'][0, :, :, :]
            # print var[:].shape


def fill_netcdf_level_tstep(nc, tstep, level, data):
    # TODO Documentation
    import numpy as np
    from timeit import default_timer as gettime

    st_time = gettime()

    # print nc
    print '\t\t\tFilling time step number {0}, level {1}'.format(tstep, level)
    for pollutant in data:
        if pollutant['data'] is not 0:
            # Condition to do anything if is an empty layer
            if float(round(pollutant['data'].sum(), 5)) != float(round(0, 5)):
                var = nc.variables[pollutant['name']]
                # print 'SUUUUUUUUUUUUUUM 0', var[:].flatten()[4425], pollutant['name'], tstep, level
                # TODO differentiate this two conditions
                if pollutant['data'].shape[1] is 1:
                    var[tstep, level, :, :] = pollutant['data'][0, 0, :, :]
                else:
                    var[tstep, level, :, :] = pollutant['data'][0, :, :]
                # print 'SUUUUUUUUUUUUUUM 1', var[:].flatten()[4425], pollutant['name'], tstep, level

    print 'TIME -> fill_netcdf_level_tstep: {0} s'.format(round(gettime() - st_time, 2))





def only_create_netcdf_old(netcdf_path, grid, data_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    compressed = False

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    # netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())
    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    # netcdf.createDimension('time', None)
    netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=compressed)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=compressed)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=compressed)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=compressed)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=compressed)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=compressed)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=compressed)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=compressed)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=compressed)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels
    # print 'DATA LIIIIST {0}'.format(data_list)
#     # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=compressed)
        var[:] = 0

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        # print "Rank {0} creating var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=compressed)
                                    # chunksizes=(1, len(levels),
                                    #             grid.x_upper_bound - grid.x_lower_bound,
                                    #             grid.y_upper_bound - grid.y_lower_bound), zlib=compressed)
        # print '1'

        var.units = Unit(variable['units']).symbol
        # print '2'
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        # print '3'
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        # print '4'
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        # print '5'
        var.coordinates = "lat lon"
        # print '6'
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'


        # print '8'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        # var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        # print "Rank {0} created var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        # c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()

def only_create_netcdf(netcdf_path, grid, data_list, levels=None, date=None, hours=None, global_attributes=None):
    from cf_units import Unit, encode_time

    # boundary_latitudes=self.grid.boundary_latitudes,
    # boundary_longitudes=self.grid.boundary_longitudes,
    # cell_area=self.grid.cell_area,
    #  RegularLatLon=True

    compressed = False
    RegularLatLon = False
    Rotated = False
    LambertConformalConic = False
    if grid.grid_type == 'global':
        RegularLatLon = True
    elif grid.grid_type == 'rotated':
        Rotated = True
    elif grid.grid_type == 'lcc':
        LambertConformalConic = True

    # netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())
    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")
    # print 'NETCDF PATH: {0}'.format(netcdf_path)

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if grid.rlat is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(grid.rlat))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if grid.rlon is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(grid.rlon))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if grid.boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(grid.boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    # netcdf.createDimension('time', None)
    netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if settings.rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=compressed)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if settings.rank == 0:
        lats[:] = grid.center_latitudes

    if grid.boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=compressed)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if settings.rank == 0:
            lat_bnds[:] = grid.boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=compressed)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if settings.rank == 0:
        lons[:] = grid.center_longitudes

    if grid.boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=compressed)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if settings.rank == 0:
            lon_bnds[:] = grid.boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=compressed)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if settings.rank == 0:
            rlat[:] = grid.rlat

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=compressed)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if settings.rank == 0:
            rlon[:] = grid.rlon
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=compressed)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if settings.rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=compressed)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if settings.rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=compressed)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if settings.rank == 0:
            lev[:] = levels
    # print 'DATA LIIIIST {0}'.format(data_list)
#     # All variables
    if len(data_list) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=compressed)
        var[:] = 0

    index = 0
    for variable in data_list:
        st_time = gettime()
        index += 1
        # print "Rank {0} creating var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=compressed)
                                    # chunksizes=(1, len(levels),
                                    #             grid.x_upper_bound - grid.x_lower_bound,
                                    #             grid.y_upper_bound - grid.y_lower_bound), zlib=compressed)
        # print '1'

        var.units = Unit(variable['units']).symbol
        # print '2'
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        # print '3'
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        # print '4'
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        # print '5'
        var.coordinates = "lat lon"
        # print '6'
        if grid.cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'


        # print '8'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        # var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        # print "Rank {0} created var: {1}; time: {2}".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = grid.new_pole_latitude_degrees
        mapping.grid_north_pole_longitude = 90 - grid.new_pole_longitude_degrees
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if grid.cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        # c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()


# @profile
def only_write_netcdf_parallel(netcdf_path, grid, var_names, emision_list):

    # print "Rank {0} 1".format(rank)

    netcdf = Dataset(netcdf_path, mode='a', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())
    index = 0
    # print "Rank {0} 2".format(rank)
    for variable in var_names:

        data = calculate_data_by_var(variable, emision_list, grid.shape)
        st_time = gettime()
        index += 1
        print "Writing var {1} Rank {0} {2}/{3} ".format(settings.rank, variable, index, len(var_names))

        var = netcdf.variables[variable]
        var.set_collective(True)
        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = data

        # print "Rank {0} ---> 2".format(rank)
        # settings.comm.Barrier()
        print "TIME -> Written var {1} Rank {0} {2} s\n".format(settings.rank, variable, round(gettime() - st_time, 2))

    if grid.cell_area is not None:
        c_area = netcdf.variables['cell_area']
        c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = grid.cell_area

    netcdf.close()


def serial_netcdf(netcdf_path, grid, data_list, emision_list, levels=None, date=None, hours=None, global_attributes=None):
    from cf_units import Unit, encode_time

    mpi_numpy = False
    mpi_vector = True

    # Gathering the index
    if mpi_numpy or mpi_vector:
        rank_position = np.array([grid.x_lower_bound, grid.x_upper_bound, grid.y_lower_bound, grid.y_upper_bound], dtype='i')
        full_position = None
        if settings.rank == 0:
            full_position = np.empty([settings.size, 4], dtype='i')
        settings.comm.Gather(rank_position, full_position, root=0)
        # print 'Rank {0} recv: {1} '.format(settings.rank, full_position)
    # exit()
    # if rank != 0:
    #
    #     for variable in data_list:
    #         rank_data = calculate_data_by_var(variable['name'], emision_list, grid.shape)
    #         settings.comm.gather(rank_data, root=0)

    if settings.rank == 0:
        compressed = True

        RegularLatLon = False
        Rotated = False
        LambertConformalConic = False

        LambertConformalConic = False
        if grid.grid_type == 'global':
            RegularLatLon = True
        elif grid.grid_type == 'rotated':
            Rotated = True
        elif grid.grid_type == 'lcc':
            LambertConformalConic = True


        netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")
        # print 'NETCDF PATH: {0}'.format(netcdf_path)

        # ===== Dimensions =====
        if RegularLatLon:
            var_dim = ('lat', 'lon',)

            # Latitude
            if len(grid.center_latitudes.shape) == 1:
                netcdf.createDimension('lat', grid.center_latitudes.shape[0])
                lat_dim = ('lat',)
            elif len(grid.center_latitudes.shape) == 2:
                netcdf.createDimension('lat', grid.center_latitudes.shape[0])
                lat_dim = ('lon', 'lat', )
            else:
                print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
                sys.exit(1)

            # Longitude
            if len(grid.center_longitudes.shape) == 1:
                netcdf.createDimension('lon', grid.center_longitudes.shape[0])
                lon_dim = ('lon',)
            elif len(grid.center_longitudes.shape) == 2:
                netcdf.createDimension('lon', grid.center_longitudes.shape[1])
                lon_dim = ('lon', 'lat', )
            else:
                print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
                sys.exit(1)
        elif Rotated:
            var_dim = ('rlat', 'rlon',)

            # Rotated Latitude
            if grid.rlat is None:
                print 'ERROR: For rotated grids is needed the rotated latitudes.'
                sys.exit(1)
            netcdf.createDimension('rlat', len(grid.rlat))
            lat_dim = ('rlat', 'rlon',)

            # Rotated Longitude
            if grid.rlon is None:
                print 'ERROR: For rotated grids is needed the rotated longitudes.'
                sys.exit(1)
            netcdf.createDimension('rlon', len(grid.rlon))
            lon_dim = ('rlat', 'rlon',)

        elif LambertConformalConic:
            var_dim = ('y', 'x',)

            netcdf.createDimension('y', len(lcc_y))
            lat_dim = ('y', 'x', )

            netcdf.createDimension('x', len(lcc_x))
            lon_dim = ('y', 'x', )

        # Levels
        if levels is not None:
            netcdf.createDimension('lev', len(levels))

        # Bounds
        if grid.boundary_latitudes is not None:
            # print boundary_latitudes.shape
            # print len(boundary_latitudes[0, 0])
            netcdf.createDimension('nv', len(grid.boundary_latitudes[0, 0]))
            # sys.exit()

        # Time
        # netcdf.createDimension('time', None)
        netcdf.createDimension('time', len(hours))

        # ===== Variables =====
        # Time
        if date is None:
            time = netcdf.createVariable('time', 'd', ('time',))
            time.units = "months since 2000-01-01 00:00:00"
            time.standard_name = "time"
            time.calendar = "gregorian"
            time.long_name = "time"
            time[:] = [0.]
        else:
            time = netcdf.createVariable('time', 'd', ('time',))
            u = Unit('hours')
            # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
            # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
            time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
            time.standard_name = "time"
            time.calendar = "gregorian"
            time.long_name = "time"
            time[:] = hours

        # Latitude
        lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=compressed)
        lats.units = "degrees_north"
        lats.axis = "Y"
        lats.long_name = "latitude coordinate"
        lats.standard_name = "latitude"
        lats[:] = grid.center_latitudes

        if grid.boundary_latitudes is not None:
            lats.bounds = "lat_bnds"
            lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=compressed)
            # print lat_bnds[:].shape, boundary_latitudes.shape
            lat_bnds[:] = grid.boundary_latitudes

        # Longitude
        lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=compressed)
        lons.units = "degrees_east"
        lons.axis = "X"
        lons.long_name = "longitude coordinate"
        lons.standard_name = "longitude"
        lons[:] = grid.center_longitudes

        if grid.boundary_longitudes is not None:
            lons.bounds = "lon_bnds"
            lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=compressed)
            # print lon_bnds[:].shape, boundary_longitudes.shape
            lon_bnds[:] = grid.boundary_longitudes

        if Rotated:
            # Rotated Latitude
            rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=compressed)
            rlat.long_name = "latitude in rotated pole grid"
            rlat.units = Unit("degrees").symbol
            rlat.standard_name = "grid_latitude"
            rlat[:] = grid.rlat

            # Rotated Longitude
            rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=compressed)
            rlon.long_name = "longitude in rotated pole grid"
            rlon.units = Unit("degrees").symbol
            rlon.standard_name = "grid_longitude"
            rlon[:] = grid.rlon
        if LambertConformalConic:
            x = netcdf.createVariable('x', 'd', ('x',), zlib=compressed)
            x.units = Unit("km").symbol
            x.long_name = "x coordinate of projection"
            x.standard_name = "projection_x_coordinate"
            x[:] = lcc_x

            y = netcdf.createVariable('y', 'd', ('y',), zlib=compressed)
            y.units = Unit("km").symbol
            y.long_name = "y coordinate of projection"
            y.standard_name = "projection_y_coordinate"
            y[:] = lcc_y

        cell_area_dim = var_dim
        # Levels
        if levels is not None:
            var_dim = ('lev',) + var_dim
            lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=compressed)
            lev.units = Unit("m").symbol
            lev.positive = 'up'
            lev[:] = levels
        # print 'DATA LIIIIST {0}'.format(data_list)
    #     # All variables
        if len(data_list) is 0:
            var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=compressed)
            var[:] = 0

    full_shape = None
    index = 0
    for variable in data_list:
        rank_data = calculate_data_by_var(variable['name'], emision_list, grid.shape)
        if mpi_numpy or mpi_vector:
            if rank_data is not None:
                root_shape = settings.comm.bcast(rank_data.shape, root=0)
                if full_shape is None:
                    # rank_shape = rank_data.shape
                    # full_shape = settings.comm.gather(rank_data.shape, root=0)
                    full_shape = settings.comm.allgather(rank_data.shape)
                    # print 'Rank {0} full_shape: {1}\n'.format(settings.rank, full_shape)
        if mpi_numpy:
            if settings.size != 1:
                if settings.rank == 0:
                    recvbuf = np.empty((settings.size,) + rank_data.shape)
                else:
                    recvbuf = None
                if root_shape != rank_data.shape:
                    rank_data_aux = np.empty(root_shape)
                    rank_data_aux[:, :, :, :-1] = rank_data
                    rank_data = rank_data_aux
                print 'Rank {0} data.shape {1}'.format(settings.rank, rank_data.shape)
                settings.comm.Gather(rank_data, recvbuf, root=0)
            else:
                recvbuf = rank_data
        elif mpi_vector:
            if rank_data is not None:
                counts_i = tuple_to_index(full_shape)
                rank_buff = [rank_data, counts_i[settings.rank]]
                if settings.rank == 0:
                    displacements = calculate_displacements(counts_i)
                    recvdata = np.empty(sum(counts_i), dtype=settings.precision)
                else:
                    displacements = None
                    recvdata = None
                if settings.precision == np.float32:
                    recvbuf = [recvdata, counts_i, displacements, MPI.FLOAT]
                elif settings.precision == np.float64:
                    recvbuf = [recvdata, counts_i, displacements, MPI.DOUBLE]
                else:
                    print 'ERROR: precission {0} unknown'.format(settings.precision)

                # print "Rank {0} sendbuf: {1}".format(settings.rank, sendbuf)

                settings.comm.Gatherv(rank_buff, recvbuf, root=0)

        else:
            if settings.size != 1:
                data = settings.comm.gather(rank_data, root=0)
            else:
                data = rank_data

        # print "Rank {0} creating var: {1}; {2}/{3}".format(rank, variable['name'], index, len(data_list))
        # print ('time',) + var_dim
        # data = calculate_data_by_var(variable['name'], emision_list, grid.shape)

        if settings.rank == 0:
            print full_shape
            # print recvbuf.shape
            if not (mpi_numpy or mpi_vector):
                if settings.size != 1:
                    try:
                        data = np.concatenate(data, axis=3)
                    except:
                        print 'var: {0} data: {1}'.format(variable['name'], data)
                        data = 0
            # print 'data shape 0 : {0}'.format(np.concatenate(data, axis=0).shape)
            # print 'data shape 1 : {0}'.format(np.concatenate(data, axis=1).shape)
            # print 'data shape 2 : {0}'.format(np.concatenate(data, axis=2).shape)
            # print 'data shape 3 : {0}'.format(np.concatenate(data, axis=3).shape)
            # exit()

            # data = fix_gathered_data(data, (len(hours), len(levels), ))
            st_time = gettime()
            index += 1
            var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=compressed)
                                        # chunksizes=(1, len(levels),
                                        #             grid.x_upper_bound - grid.x_lower_bound,
                                        #             grid.y_upper_bound - grid.y_lower_bound), zlib=compressed)
            # print '1'

            var.units = Unit(variable['units']).symbol
            # print '2'
            if 'long_name' in variable:
                var.long_name = str(variable['long_name'])
            # print '3'
            if 'standard_name' in variable:
                var.standard_name = str(variable['standard_name'])
            # print '4'
            if 'cell_method' in variable:
                var.cell_method = str(variable['cell_method'])
            # print '5'
            var.coordinates = "lat lon"
            # print '6'
            if grid.cell_area is not None:
                var.cell_measures = 'area: cell_area'
            if RegularLatLon:
                var.grid_mapping = 'crs'
            elif Rotated:
                var.grid_mapping = 'rotated_pole'
            elif LambertConformalConic:
                var.grid_mapping = 'Lambert_conformal'


            # print '8'
            # if variable['data'] is not 0:
            #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
            # print variable
            #
            if mpi_numpy:
                data = np.ones(var[:].shape, dtype=settings.precision) * 100
                for i in xrange(settings.size):
                    print i, full_shape[i]
                    # print recvbuf[i, :, :, :full_shape[i][-2], : full_shape[i][-1]].shape
                    # print i
                    if True:
                        try:
                            if i == 0:
                                # print full_position[i]
                                # data[:, :, :, :full_position[i][3]] = recvbuf[i]
                                var[:, :, :, :full_position[i][3]] = recvbuf[i]
                            elif i == settings.size - 1:
                                # data[:, :, :, full_position[i][2]:] = recvbuf[i, :, :, :, : full_shape[i][-1]]
                                # data[:, :, :, full_position[i][2]:] = recvbuf[i, :, :, :, :-1 ]
                                var[:, :, :, full_position[i][2]:] = recvbuf[i, :, :, :, :-1]
                            else:
                                # data[:, :, :, full_position[i][2]:full_position[i][3]] = recvbuf[i, :, :, :, : full_shape[i][-1]]
                                var[:, :, :, full_position[i][2]:full_position[i][3]] = recvbuf[i, :, :, :,
                                                                                         : full_shape[i][-1]]
                                # data[:, :, :, full_position[i][2]:full_position[i][3]] = recvbuf[i, :, :, :,1:]
                            # data[:, :, :, full_position[i][2]:full_position[i][3]] = recvbuf[i, :, :, :, : full_shape[i][-1]]
                        except:
                            print 'ERROR on i {0}'.format(i)
                            print 'data shape: {0} recvbuf shape {1}'.format(data[:, :, :, full_position[i][2]:].shape, recvbuf[i].shape)
                            # print 'data shape: {0} recvbuf shape {1}'.format(data[:, :, :, full_position[i][2]:full_position[i][3]].shape, recvbuf[i, :, :, :, : full_shape[i][-1]].shape)


                    # data[:, :, full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = recvbuf[i, :, :, :full_shape[i][-2] +1, : full_shape[i][-1]+1]
                    # var[:, :, full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = recvbuf[i, :, :, :full_shape[i][-2], : full_shape[i][-1]]
                # print data[:] - var[:]
                # var[:] = data
            elif mpi_vector:
                var_time = gettime()

                # data_list = []#np.empty(shape, dtype=np.float64)

                if rank_data is not None:
                    data = np.empty(var[:].shape, dtype=settings.precision)
                    for i in xrange(settings.size):
                        # print 'Resizeing {0}'.format(i)
                        if not i == settings.size - 1:
                            # print dspls_i[i], dspls_i[i + 1], recvbuf[0][0:9], counts[i]
                            # data_aux = np.array(recvbuf[0][dspls_i[i]: dspls_i[i + 1]]).reshape(counts[i])
                            data[:, :, full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = np.array(recvbuf[0][displacements[i]: displacements[i + 1]]).reshape(full_shape[i])
                        else:
                            # data_aux = np.array(recvbuf[0][dspls_i[i]:]).reshape(counts[i])
                            data[:, :, full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = np.array(recvbuf[0][displacements[i]:]).reshape(full_shape[i])
                    #     data_list.append(data_aux)
                    # data = np.concatenate(data_list, axis=1)
                else:
                    data = 0
                print "Var {0} comm time: {1}".format(variable['name'], round(gettime() - var_time, 2))
                var[:] = data
            else:
                var[:] = data
            print "Var {0} writing time: {1}".format(variable['name'], round(gettime() - st_time, 2))
    # exit()


    if settings.rank == 0:
        # Grid mapping
        if RegularLatLon:
            # CRS
            mapping = netcdf.createVariable('crs', 'i')
            mapping.grid_mapping_name = "latitude_longitude"
            mapping.semi_major_axis = 6371000.0
            mapping.inverse_flattening = 0
        elif Rotated:
            # Rotated pole
            mapping = netcdf.createVariable('rotated_pole', 'c')
            mapping.grid_mapping_name = 'rotated_latitude_longitude'
            mapping.grid_north_pole_latitude = grid.new_pole_latitude_degrees
            mapping.grid_north_pole_longitude = 90 - grid.new_pole_longitude_degrees
        elif LambertConformalConic:
            # CRS
            mapping = netcdf.createVariable('Lambert_conformal', 'i')
            mapping.grid_mapping_name = "lambert_conformal_conic"
            mapping.standard_parallel = lat_1_2
            mapping.longitude_of_central_meridian = lon_0
            mapping.latitude_of_projection_origin = lat_0

    if grid.cell_area is not None:
        # counts_i = tuple_to_index(full_shape, bidimensional=True)
        # # print grid.cell_area.shape
        # rank_buff = [np.ascontiguousarray(grid.cell_area, dtype=np.float32), counts_i[settings.rank]]
        # if settings.rank == 0:
        #     displacements = calculate_displacements(counts_i)
        #     recvdata = np.empty(sum(counts_i), dtype=settings.precision)
        # else:
        #     displacements = None
        #     recvdata = None
        # recvbuf = [recvdata, counts_i, displacements, MPI.DOUBLE]
        #
        # # print "Rank {0} sendbuf: {1}".format(settings.rank, sendbuf)
        #
        # settings.comm.Gatherv(rank_buff, recvbuf, root=0)

        cell_area = settings.comm.gather(grid.cell_area, root=0)
        # cell_area = np.concatenate(cell_area, axis=1)
        if settings.rank == 0:
            # Cell area
            if grid.cell_area is not None:
                c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
                c_area.long_name = "area of the grid cell"
                c_area.standard_name = "cell_area"
                c_area.units = Unit("m2").symbol
                # print c_area[:].shape, cell_area.shape
                # c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

                # cell_area = np.zeros(c_area[:].shape, dtype=settings.precision)
                # # for i in xrange(settings.size):
                # for i in xrange(1):
                #     print recvbuf[0][:10]
                #     print np.ascontiguousarray(grid.cell_area, dtype=np.float32).flatten()[:10]
                #     # print 'Resizeing {0}'.format(i)
                #     print full_shape[i]
                #     if not i == settings.size - 1:
                #         # print dspls_i[i], dspls_i[i + 1], recvbuf[0][0:9], counts[i]
                #         # data_aux = np.array(recvbuf[0][dspls_i[i]: dspls_i[i + 1]]).reshape(counts[i])
                #         cell_area[full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = np.array(recvbuf[0][displacements[i]: displacements[i + 1]]).reshape((full_shape[i][-2], full_shape[i][-1]))
                #         # cell_area[full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = grid.cell_area
                #     else:
                #         # data_aux = np.array(recvbuf[0][dspls_i[i]:]).reshape(counts[i])
                #         cell_area[full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = np.array(recvbuf[0][displacements[i]:]).reshape((full_shape[i][-2], full_shape[i][-1]))
                #         # cell_area[full_position[i][0]:full_position[i][1], full_position[i][2]:full_position[i][3]] = grid.cell_area
                #
                cell_area = np.concatenate(cell_area, axis=1)
                c_area[:] = cell_area

            if global_attributes is not None:
                netcdf.setncatts(global_attributes)

            netcdf.close()

def only_create_write_netcdf(netcdf_path, grid, var_names, emision_list,
                 levels=None, date=None, hours=None,
                 boundary_latitudes=None, boundary_longitudes=None, cell_area=None, global_attributes=None,
                 RegularLatLon=False,
                 Rotated=False, rotated_lats=None, rotated_lons=None, north_pole_lat=None, north_pole_lon=None,
                 LambertConformalConic=False, lcc_x=None, lcc_y=None, lat_1_2=None, lon_0=None, lat_0=None):
    """

    :param netcdf_path:
    :param center_latitudes:
    :param center_longitudes:
    :param data_list:
    :param levels:
    :param boundary_latitudes:
    :param boundary_longitudes:
    :param cell_area:

    :param global_attributes:
    :type global_attributes: dict

    :param RegularLatLon:
    :param Rotated:
    :param rotated_lats:
    :param rotated_lons:
    :param north_pole_lat:
    :param north_pole_lon:
    :param LambertConformalConic:
    :return:
    """
    from cf_units import Unit, encode_time

    compressed = False

    if not (RegularLatLon or LambertConformalConic or Rotated):
        RegularLatLon = True

    netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4", parallel=True, comm=settings.comm, info=MPI.Info())
    # netcdf = Dataset(netcdf_path, mode='w', format="NETCDF4")

    # ===== Dimensions =====
    if RegularLatLon:
        var_dim = ('lat', 'lon',)

        # Latitude
        if len(grid.center_latitudes.shape) == 1:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lat',)
        elif len(grid.center_latitudes.shape) == 2:
            netcdf.createDimension('lat', grid.center_latitudes.shape[0])
            lat_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Latitudes must be on a 1D or 2D array instead of {0}'.format(len(center_latitudes.shape))
            sys.exit(1)

        # Longitude
        if len(grid.center_longitudes.shape) == 1:
            netcdf.createDimension('lon', grid.center_longitudes.shape[0])
            lon_dim = ('lon',)
        elif len(grid.center_longitudes.shape) == 2:
            netcdf.createDimension('lon', grid.center_longitudes.shape[1])
            lon_dim = ('lon', 'lat', )
        else:
            print 'ERROR: Longitudes must be on a 1D or 2D array instead of {0}'.format(len(center_longitudes.shape))
            sys.exit(1)
    elif Rotated:
        var_dim = ('rlat', 'rlon',)

        # Rotated Latitude
        if rotated_lats is None:
            print 'ERROR: For rotated grids is needed the rotated latitudes.'
            sys.exit(1)
        netcdf.createDimension('rlat', len(rotated_lats))
        lat_dim = ('rlat', 'rlon',)

        # Rotated Longitude
        if rotated_lons is None:
            print 'ERROR: For rotated grids is needed the rotated longitudes.'
            sys.exit(1)
        netcdf.createDimension('rlon', len(rotated_lons))
        lon_dim = ('rlat', 'rlon',)

    elif LambertConformalConic:
        var_dim = ('y', 'x',)

        netcdf.createDimension('y', len(lcc_y))
        lat_dim = ('y', 'x', )

        netcdf.createDimension('x', len(lcc_x))
        lon_dim = ('y', 'x', )

    # Levels
    if levels is not None:
        netcdf.createDimension('lev', len(levels))

    # Bounds
    if boundary_latitudes is not None:
        # print boundary_latitudes.shape
        # print len(boundary_latitudes[0, 0])
        netcdf.createDimension('nv', len(boundary_latitudes[0, 0]))
        # sys.exit()

    # Time
    netcdf.createDimension('time', None)
    # netcdf.createDimension('time', len(hours))

    # ===== Variables =====
    # Time
    if date is None:
        time = netcdf.createVariable('time', 'd', ('time',))
        time.units = "months since 2000-01-01 00:00:00"
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        time[:] = [0.]
    else:
        time = netcdf.createVariable('time', 'd', ('time',))
        u = Unit('hours')
        # print u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second))
        # Unit('hour since 1970-01-01 00:00:00.0000000 UTC')
        time.units = str(u.offset_by_time(encode_time(date.year, date.month, date.day, date.hour, date.minute, date.second)))
        time.standard_name = "time"
        time.calendar = "gregorian"
        time.long_name = "time"
        if rank == 0:
            time[:] = hours

    # Latitude
    lats = netcdf.createVariable('lat', 'f', lat_dim, zlib=compressed)
    lats.units = "degrees_north"
    lats.axis = "Y"
    lats.long_name = "latitude coordinate"
    lats.standard_name = "latitude"
    if rank == 0:
        lats[:] = grid.center_latitudes

    if boundary_latitudes is not None:
        lats.bounds = "lat_bnds"
        lat_bnds = netcdf.createVariable('lat_bnds', 'f', lat_dim + ('nv',), zlib=compressed)
        # print lat_bnds[:].shape, boundary_latitudes.shape
        if rank == 0:
            lat_bnds[:] = boundary_latitudes

    # Longitude
    lons = netcdf.createVariable('lon', 'f', lon_dim, zlib=compressed)
    lons.units = "degrees_east"
    lons.axis = "X"
    lons.long_name = "longitude coordinate"
    lons.standard_name = "longitude"
    if rank == 0:
        lons[:] = grid.center_longitudes

    if boundary_longitudes is not None:
        lons.bounds = "lon_bnds"
        lon_bnds = netcdf.createVariable('lon_bnds', 'f', lon_dim + ('nv',), zlib=compressed)
        # print lon_bnds[:].shape, boundary_longitudes.shape
        if rank == 0:
            lon_bnds[:] = boundary_longitudes

    if Rotated:
        # Rotated Latitude
        rlat = netcdf.createVariable('rlat', 'f', ('rlat',), zlib=compressed)
        rlat.long_name = "latitude in rotated pole grid"
        rlat.units = Unit("degrees").symbol
        rlat.standard_name = "grid_latitude"
        if rank == 0:
            rlat[:] = rotated_lats

        # Rotated Longitude
        rlon = netcdf.createVariable('rlon', 'f', ('rlon',), zlib=compressed)
        rlon.long_name = "longitude in rotated pole grid"
        rlon.units = Unit("degrees").symbol
        rlon.standard_name = "grid_longitude"
        if rank == 0:
            rlon[:] = rotated_lons
    if LambertConformalConic:
        x = netcdf.createVariable('x', 'd', ('x',), zlib=compressed)
        x.units = Unit("km").symbol
        x.long_name = "x coordinate of projection"
        x.standard_name = "projection_x_coordinate"
        if rank == 0:
            x[:] = lcc_x

        y = netcdf.createVariable('y', 'd', ('y',), zlib=compressed)
        y.units = Unit("km").symbol
        y.long_name = "y coordinate of projection"
        y.standard_name = "projection_y_coordinate"
        if rank == 0:
            y[:] = lcc_y

    cell_area_dim = var_dim
    # Levels
    if levels is not None:
        var_dim = ('lev',) + var_dim
        lev = netcdf.createVariable('lev', 'f', ('lev',), zlib=compressed)
        lev.units = Unit("m").symbol
        lev.positive = 'up'
        if rank == 0:
            lev[:] = levels
    # print 'DATA LIIIIST {0}'.format(data_list)
#     # All variables
    if len(var_names) is 0:
        var = netcdf.createVariable('aux_var', 'f', ('time',) + var_dim, zlib=compressed)
        var[:] = 0

    index = 0
    for variable in var_names:
        st_time = gettime()
        index += 1
        print "Rank {0} creating var: {1}; {2}/{3}".format(rank, variable['name'], index, len(var_names))
        # print ('time',) + var_dim
        var = netcdf.createVariable(variable['name'], 'f', ('time',) + var_dim, zlib=compressed)
        # print '1'

        var.units = Unit(variable['units']).symbol
        # print '2'
        if 'long_name' in variable:
            var.long_name = str(variable['long_name'])
        # print '3'
        if 'standard_name' in variable:
            var.standard_name = str(variable['standard_name'])
        # print '4'
        if 'cell_method' in variable:
            var.cell_method = str(variable['cell_method'])
        # print '5'
        var.coordinates = "lat lon"
        # print '6'
        if cell_area is not None:
            var.cell_measures = 'area: cell_area'
        if RegularLatLon:
            var.grid_mapping = 'crs'
        elif Rotated:
            var.grid_mapping = 'rotated_pole'
        elif LambertConformalConic:
            var.grid_mapping = 'Lambert_conformal'

        var.set_collective(True)

        data = calculate_data_by_var(variable['name'], emision_list, grid.shape)

        var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = data

        # print '8'
        # if variable['data'] is not 0:
        #     print '{0} {1}'.format(var[:, :, grid.x_lower_bound:grid.x_upper_bound,  grid.y_lower_bound:grid.y_upper_bound].shape, variable['data'].shape)
        # print variable

        # var[:, :, grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = variable['data']

        print "Rank {0} created var: {1}; time: {2} s".format(rank, variable['name'], round(gettime() - st_time, 2))

    # Grid mapping
    if RegularLatLon:
        # CRS
        mapping = netcdf.createVariable('crs', 'i')
        mapping.grid_mapping_name = "latitude_longitude"
        mapping.semi_major_axis = 6371000.0
        mapping.inverse_flattening = 0
    elif Rotated:
        # Rotated pole
        mapping = netcdf.createVariable('rotated_pole', 'c')
        mapping.grid_mapping_name = 'rotated_latitude_longitude'
        mapping.grid_north_pole_latitude = north_pole_lat
        mapping.grid_north_pole_longitude = north_pole_lon
    elif LambertConformalConic:
        # CRS
        mapping = netcdf.createVariable('Lambert_conformal', 'i')
        mapping.grid_mapping_name = "lambert_conformal_conic"
        mapping.standard_parallel = lat_1_2
        mapping.longitude_of_central_meridian = lon_0
        mapping.latitude_of_projection_origin = lat_0

    # Cell area
    if cell_area is not None:
        c_area = netcdf.createVariable('cell_area', 'f', cell_area_dim)
        c_area.long_name = "area of the grid cell"
        c_area.standard_name = "cell_area"
        c_area.units = Unit("m2").symbol
        # print c_area[:].shape, cell_area.shape
        # c_area[grid.x_lower_bound:grid.x_upper_bound, grid.y_lower_bound:grid.y_upper_bound] = cell_area

    if global_attributes is not None:
        netcdf.setncatts(global_attributes)

    netcdf.close()

# @profile
def calculate_data_by_var(variable, inventory_list, shape):
    # TODO Documentation
    """

    :param variable:
    :param inventory_list:
    :param shape:
    :return:
    """
    from timeit import default_timer as gettime

    st_time = gettime()
    # print 'Getting Data'
    # print variable, shape

    data = None
    # data = np.zeros(shape)

    for ei in inventory_list:
        for emission in ei.emissions:
            if emission['name'] == variable:
                if emission['data'] is not 0:
                    # print variable

                    if ei.vertical_factors is not None:
                        aux_data = emission['data'] * ei.vertical_factors[:, np.newaxis, np.newaxis]
                    else:
                        if not len(emission['data'].shape) == 4:
                            aux_data = np.zeros((shape[1], shape[2], shape[3]))
                            aux_data[0, :, :] = emission['data']
                        else:
                            aux_data = emission['data']

                    del emission['data']

                    if ei.temporal_factors is not None:
                        if data is None:
                            data = aux_data[np.newaxis, :, :, :] * ei.temporal_factors[:, np.newaxis, :, :]
                        else:
                            data += aux_data[np.newaxis, :, :, :] * ei.temporal_factors[:, np.newaxis, :, :]
                    else:
                        if data is None:
                            data = aux_data[np.newaxis, :, :, :]
                        else:
                            data += aux_data[np.newaxis, :, :, :]

    print "TIME -> get_data_by_var: Rank {2} {0} -> {1} s\n".format(variable, round(gettime() - st_time, 4), settings.rank)

    return data


def tuple_to_index(tuple_list, bidimensional=False):
    from operator import mul
    new_list = []
    for tuple in tuple_list:
        if bidimensional:
            new_list.append(tuple[-1] * tuple[-2])
        else:
            new_list.append(reduce(mul, tuple))
    return new_list


def calculate_displacements(counts):
    new_list = [0]
    accum = 0
    for counter in counts[:-1]:
        accum += counter
        new_list.append(accum)
    return new_list


if __name__ == '__main__':
    pass




